<?php

/**
 * @file
 * Main template file.
 *
 * Implements preprocess, process functions and alter hooks.
 */

// Include drupal theme and helper functions.
require_once dirname(__FILE__) . '/includes/utils.inc';
require_once dirname(__FILE__) . '/includes/breadcrumb.inc';
require_once dirname(__FILE__) . '/includes/form.inc';
require_once dirname(__FILE__) . '/includes/menu.inc';
require_once dirname(__FILE__) . '/includes/message.inc';
require_once dirname(__FILE__) . '/includes/item_list.inc';
require_once dirname(__FILE__) . '/includes/pager.inc';
require_once dirname(__FILE__) . '/includes/links.inc';
require_once dirname(__FILE__) . '/includes/status_report.inc';

/**
 * Implements hook_theme_registry_alter().
 */
function semanticui_theme_registry_alter(&$registry) {
  global $theme_key;
  $available_themes = list_themes();
  $base_themes = array();
  if (isset($available_themes[$theme_key]->info['base theme'])) {
    $base_themes = drupal_find_base_themes($available_themes, $theme_key);
  }
  // Use semanticui theme as it provides necessary
  // functionality for page elements.
  $base_themes += array('semanticui' => 'semanticui');
  $base_themes += array($theme_key => $theme_key);

  foreach ($base_themes as $theme_key => $theme_human_name) {
    if (empty($theme_key)) {
      continue;
    }
    semanticui_discover_hooks($theme_key, $registry);
  }
}

/**
 * Discovers and registers preprocess, process theme functions.
 */
function semanticui_discover_hooks($theme, &$registry) {
  foreach (array('process', 'preprocess') as $type) {
    // Iterate over all preprocess/process files in the current theme.
    $discovered_files = semanticui_discover_files($theme, $type);
    foreach ($discovered_files as $item) {
      $callback = "{$theme}_{$type}_{$item->hook}";

      // If there is no hook with that name, continue.
      if (!array_key_exists($item->hook, $registry)) {
        continue;
      }

      // Append the included (pre-)process hook to the array of functions.
      $registry[$item->hook]["$type functions"][] = $callback;

      // By adding this file to the 'includes' array we make sure that it is
      // available when the hook is executed.
      $registry[$item->hook]['includes'][] = $item->uri;
    }
  }
}

/**
 * Scans for files of a certain type in the current theme's path.
 */
function semanticui_discover_files($theme, $type) {
  $length = -(strlen($type) + 1);

  $path = drupal_get_path('theme', $theme);
  // Only look for files that match the 'something.preprocess.inc' pattern.
  $mask = '/.' . $type . '.inc$/';

  // Recursively scan the folder for the current step for (pre-)process
  // files and write them to the registry.
  $files = file_scan_directory($path . '/' . $type, $mask);
  foreach ($files as &$file) {
    $file->hook = strtr(substr($file->name, 0, $length), '-', '_');
  }

  return $files;
}

/**
 * Implements hook_css_alter().
 */
function semanticui_css_alter(&$css) {
  $exclude = array(
    'modules/aggregator/aggregator.css' => FALSE,
    'modules/comment/comment.css' => FALSE,
    'modules/system/system.css' => FALSE,
    'modules/system/system.menus.css' => FALSE,
    'modules/system/system.messages.css' => FALSE,
    'modules/system/system.theme.css' => FALSE,
    'modules/user/user.css' => FALSE,
    'modules/search/search.css' => FALSE,
    'modules/filter/filter.css' => FALSE,
    'modules/field/theme/field.css' => FALSE,
    'modules/forum/forum.css' => FALSE,
    'misc/vertical-tabs.css' => FALSE,
  );
  $css = array_diff_key($css, $exclude);
}

/**
 * Implements hook_js_alter().
 */
function semanticui_js_alter(&$js) {
  if (array_key_exists('misc/tabledrag.js', $js)) {
    drupal_add_js(drupal_get_path('theme', 'semanticui') . '/js/tabledrag.js');
  }
  if (array_key_exists('modules/user/user.js', $js)) {
    drupal_add_js(drupal_get_path('theme', 'semanticui') . '/js/user.js');
  }
  if (array_key_exists('misc/ajax.js', $js)) {
    // It would be good to alter ajax library and add custom file in the
    // hook_library_alter but it provides fully populated array with libraries
    // only at the module level implementation.
    drupal_add_js(drupal_get_path('theme', 'semanticui') . '/js/ajax.js');
  }
  if (array_key_exists('modules/user/user.permissions.js', $js)) {
    drupal_add_js(drupal_get_path('theme', 'semanticui') . '/js/user.permissions.js');
  }

  // Add semantic js library, if it was not added in semanticui_preprocess_html().
  $semantic_js = libraries_get_path('semanticui') . '/semantic.min.js';
  if (!array_key_exists($semantic_js, $js)) {
    if (semanticui_check_library('semanticui', 'semantic.min.js')) {
      $js[$semantic_js] = drupal_js_defaults();
      $js[$semantic_js]['data'] = $semantic_js;
      $js[$semantic_js]['every_page'] = TRUE;
    }
  }
}

/**
 * Implements hook_html_head_alter().
 */
function semanticui_html_head_alter(&$head_elements) {
  // Replace the meta content-type tag to be HTML5 compatible.
  $head_elements['system_meta_content_type']['#attributes'] = array(
    'charset' => 'utf-8',
  );

  // Optimize the mobile viewport.
  $head_elements['mobile_viewport'] = array(
    '#type' => 'html_tag',
    '#tag' => 'meta',
    '#attributes' => array(
      'name' => 'viewport',
      'content' => 'width=device-width, initial-scale=1.0',
    ),
  );

  // Force the latest IE rendering engine and Google Chrome Frame.
  $head_elements['chrome_frame'] = array(
    '#type' => 'html_tag',
    '#tag' => 'meta',
    '#attributes' => array(
      'http-equiv' => 'X-UA-Compatible',
      'content' => 'IE=edge,chrome=1',
    ),
  );
}

/**
 * Implements hook_theme().
 */
function semanticui_theme($existing, $type, $theme, $path) {
  return array(
    'search_block_input_wrapper' => array(
      'render element' => 'element',
      'function' => 'semanticui_search_block_input_wrapper',
    ),
    'pager_item_list' => array(
      'variables' => array(
        'items' => array(),
      ),
    ),
    'labeled_button' => array(
      'render element' => 'element',
    ),
  );
}

/**
 * Implements hook_library_alter().
 */
function semanticui_library_alter(&$libraries, $module) {
  if ($module == 'contextual' && isset($libraries['contextual-links'])) {
    $library = drupal_get_path('theme', 'semanticui') . '/js/contextual.js';
    $libraries['contextual-links']['js'][$library] = array();
  }
}

/**
 * Implements hook_comment_view_alter().
 */
function semanticui_comment_view_alter(&$build) {
  // Delete "Log in or register to post comments" from each comment.
  if (($build['#node']->type == 'forum')) {
    if (isset($build['links']['comment']['#links']['comment_forbidden'])) {
      unset($build['links']['comment']['#links']['comment_forbidden']);
    }
  }
}

/**
 * Implements hook_element_info_alter().
 *
 * Fixes the bug with multiple inline errors for the elements group, like
 * multiple radio buttons: by default the inline error will be shown to each
 * radio button and to the buttons group.
 */
function semanticui_element_info_alter(&$type) {
  $type['radios']['#process'][] = 'semanticui_form_process_radios';
}
